@@ -2,10 +2,13 @@
 // called LICENSE at the top level of the ICU4X source tree
 // (online at: https://github.com/unicode-org/icu4x/blob/main/LICENSE ).
 
+//! The collection of code for locale canonicalization.
+
 use crate::provider::*;
 use alloc::string::ToString;
 use alloc::vec;
 use alloc::vec::Vec;
+use core::mem;
 use icu_locid::{
     extensions::unicode::{Key, Value},
     subtags, LanguageIdentifier, Locale,
@@ -16,13 +19,94 @@ use tinystr::{tinystr4, TinyStr4, TinyStr8};
 /// Used to track the result of a canonicalization operation that potentially modifies its argument in place.
 #[derive(Debug, PartialEq)]
 pub enum CanonicalizationResult {
+    /// The canonicalization operation modified the locale.
     Modified,
+    /// The canonicalization operation did not modify the locale.
     Unmodified,
 }
 
+/// LocaleCanonicalizer implementation.
+///
+/// The LocaleCanonicalizer provides methods to canonicalize Locales and
+/// LanguageIdentifiers based upon [`CLDR`] data.
+///
+/// It currently supports locale canonicalization based upon the canonicalization
+/// algorithm from [`UTS #35: Unicode LDML 3. LocaleId Canonicalization`].
+///
+/// It also supports the `minimize` and `maximize` likely subtags algorithms
+/// as described in [`UTS #35: Unicode LDML 3. Likely Subtags`].
+///
+/// The maximize method potentially updates a passed in locale in place
+/// depending up the results of running the 'Add Likely Subtags' algorithm
+/// from [`UTS #35: Unicode LDML 3. Likely Subtags`].
+///
+/// This minimize method returns a new Locale that is the result of running the
+/// 'Remove Likely Subtags' algorithm from [`UTS #35: Unicode LDML 3. Likely Subtags`].
+///
+/// # Examples
+///
+/// ```
+/// use icu_locale_canonicalizer::{CanonicalizationResult, LocaleCanonicalizer};
+/// use icu_locid::Locale;
+///
+/// let provider = icu_testdata::get_provider();
+/// let lc = LocaleCanonicalizer::new(&provider)
+///     .expect("create failed");
+///
+/// let mut locale : Locale = "ja-Latn-fonipa-hepburn-heploc".parse()
+///     .expect("parse failed");
+/// assert_eq!(lc.canonicalize(&mut locale), CanonicalizationResult::Modified);
+/// assert_eq!(locale.to_string(), "ja-Latn-alalc97-fonipa");
+/// ```
+///
+/// ```
+/// use icu_locale_canonicalizer::{CanonicalizationResult, LocaleCanonicalizer};
+/// use icu_locid::Locale;
+///
+/// let provider = icu_testdata::get_provider();
+/// let lc = LocaleCanonicalizer::new(&provider)
+///     .expect("create failed");
+///
+/// let mut locale : Locale = "zh-CN".parse()
+///     .expect("parse failed");
+/// assert_eq!(lc.maximize(&mut locale), CanonicalizationResult::Modified);
+/// assert_eq!(locale.to_string(), "zh-Hans-CN");
+///
+/// let mut locale : Locale = "zh-Hant-TW".parse()
+///     .expect("parse failed");
+/// assert_eq!(lc.maximize(&mut locale), CanonicalizationResult::Unmodified);
+/// assert_eq!(locale.to_string(), "zh-Hant-TW");
+/// ```
+///
+/// ```
+/// use icu_locale_canonicalizer::{CanonicalizationResult, LocaleCanonicalizer};
+/// use icu_locid::Locale;
+///
+/// let provider = icu_testdata::get_provider();
+/// let lc = LocaleCanonicalizer::new(&provider)
+///     .expect("create failed");
+///
+/// let mut locale : Locale = "zh-Hans-CN".parse()
+///     .expect("parse failed");
+/// assert_eq!(lc.minimize(&mut locale), CanonicalizationResult::Modified);
+/// assert_eq!(locale.to_string(), "zh");
+///
+/// let mut locale : Locale = "zh".parse()
+///     .expect("parse failed");
+/// assert_eq!(lc.minimize(&mut locale), CanonicalizationResult::Unmodified);
+/// assert_eq!(locale.to_string(), "zh");
+/// ```
+///
+/// [`ICU4X`]: ../icu/index.html
+/// [`CLDR`]: http://cldr.unicode.org/
+/// [`UTS #35: Unicode LDML 3. Likely Subtags`]: https://www.unicode.org/reports/tr35/#Likely_Subtags.
+/// [`UTS #35: Unicode LDML 3. LocaleId Canonicalization`]: http://unicode.org/reports/tr35/#LocaleId_Canonicalization,
 pub struct LocaleCanonicalizer<'data> {
+    /// Data to support canonicalization.
     aliases: DataPayload<'data, AliasesV1Marker>,
+    /// Data to support likely subtags maximize and minimize.
     likely_subtags: DataPayload<'data, LikelySubtagsV1Marker>,
+    /// Extension keys that require canonicalization.
     extension_keys: Vec<Key>,
 }
 
@@ -391,7 +475,7 @@ impl<'data> LocaleCanonicalizer<'data> {
                         .binary_search_by_key(&value_as_tinystr, |alias| alias.0)
                     {
                         if let Ok(modified_value) =
-                            Value::from_bytes(&self.aliases.get().subdivision[index].1.as_bytes())
+                            Value::from_bytes(self.aliases.get().subdivision[index].1.as_bytes())
                         {
                             *value = modified_value;
                             result = CanonicalizationResult::Modified;
@@ -496,6 +580,7 @@ impl<'data> LocaleCanonicalizer<'data> {
 
         let mut max = langid.clone();
         self.maximize(&mut max);
+        let variants = mem::take(&mut max.variants);
         max.variants.clear();
         let mut trial = max.clone();
 
@@ -503,9 +588,18 @@ impl<'data> LocaleCanonicalizer<'data> {
         trial.region = None;
         self.maximize(&mut trial);
         if trial == max {
-            if langid.script.is_some() || langid.script.is_some() {
-                langid.script = None;
-                langid.region = None;
+            if langid.language != max.language || langid.script.is_some() || langid.region.is_some()
+            {
+                if langid.language != max.language {
+                    langid.language = max.language
+                }
+                if langid.script.is_some() {
+                    langid.script = None;
+                }
+                if langid.region.is_some() {
+                    langid.region = None;
+                }
+                langid.variants = variants;
                 return CanonicalizationResult::Modified;
             } else {
                 return CanonicalizationResult::Unmodified;
@@ -516,9 +610,20 @@ impl<'data> LocaleCanonicalizer<'data> {
         trial.region = max.region;
         self.maximize(&mut trial);
         if trial == max {
-            if langid.script.is_some() || langid.region != max.region {
-                langid.script = None;
-                langid.region = max.region;
+            if langid.language != max.language
+                || langid.script.is_some()
+                || langid.region != max.region
+            {
+                if langid.language != max.language {
+                    langid.language = max.language
+                }
+                if langid.script.is_some() {
+                    langid.script = None;
+                }
+                if langid.region != max.region {
+                    langid.region = max.region;
+                }
+                langid.variants = variants;
                 return CanonicalizationResult::Modified;
             } else {
                 return CanonicalizationResult::Unmodified;
@@ -529,18 +634,39 @@ impl<'data> LocaleCanonicalizer<'data> {
         trial.region = None;
         self.maximize(&mut trial);
         if trial == max {
-            if langid.script != max.script || langid.region.is_some() {
-                langid.script = max.script;
-                langid.region = None;
+            if langid.language != max.language
+                || langid.script != max.script
+                || langid.region.is_some()
+            {
+                if langid.language != max.language {
+                    langid.language = max.language
+                }
+                if langid.script != max.script {
+                    langid.script = max.script;
+                }
+                if langid.region.is_some() {
+                    langid.region = None;
+                }
+                langid.variants = variants;
                 return CanonicalizationResult::Modified;
             } else {
                 return CanonicalizationResult::Unmodified;
             }
         }
 
-        if langid.script != max.script || langid.region != max.region {
-            langid.script = max.script;
-            langid.region = max.region;
+        if langid.language != max.language
+            || langid.script != max.script
+            || langid.region != max.region
+        {
+            if langid.language != max.language {
+                langid.language = max.language
+            }
+            if langid.script != max.script {
+                langid.script = max.script;
+            }
+            if langid.region != max.region {
+                langid.region = max.region;
+            }
             CanonicalizationResult::Modified
         } else {
             CanonicalizationResult::Unmodified
